﻿using System;
using System.Runtime.InteropServices;

/* Program.cs
   Part of "APCComm / windows Version" 
   Copyright (C) 2000,2001,2006 Ralf Hoffmann
   Original Contact: ralf@boomerangsworld.de
   Windows version Contact: evil.lombo@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/
/* $Id: Program.cs,v 1.11 2013/10/30 22:35:32 Krahen Exp $ */

namespace ParallelPort
{
	public class PortAccess
	{
		[DllImport("inpout32.dll")]
		private static extern uint IsInpOutDriverOpen();

		[DllImport("inpout32.dll")]
		private static extern void Out32(short PortAddress, short Data);
		
		[DllImport("inpout32.dll")]
		private static extern byte Inp32(short PortAddress);
		
		[DllImport("inpoutx64.dll", EntryPoint = "IsInpOutDriverOpen")]
		private static extern uint IsInpOutDriverOpen_x64();
		
		[DllImport("inpoutx64.dll", EntryPoint = "Out32")]
		private static extern void Out32_x64(short PortAddress, short Data);

		[DllImport("inpoutx64.dll", EntryPoint = "Inp32")]
		private static extern char Inp32_x64(short PortAddress);

		private short _PortAddress;
		private bool _X64;
		
		public PortAccess(short PortAddress, bool X64)
		{
			_PortAddress = PortAddress;
			_X64 = X64;
			
			try
			{
				uint nResult = 0;
				if (X64)
					nResult = IsInpOutDriverOpen_x64();
				else
					nResult = IsInpOutDriverOpen();

				if (nResult == 0)
				{
					throw new ArgumentException("Unable to open InpOut32 driver");
				}
			}
			catch (DllNotFoundException)
			{
				throw new ArgumentException("Unable to find InpOut32.dll");
			}

		}

		public void Write(byte Data)
		{
			if (_X64)
				Out32_x64(_PortAddress,(short)Data);
			else
				Out32(_PortAddress,(short)Data);
		}

		public byte Read()
		{
			if (_X64)
				return (byte)Inp32_x64(_PortAddress);
			else
				return (byte)Inp32(_PortAddress);
		}
	}
}